-- CREATE LANGUAGE plpgsql;
--DROP FUNCTION srch_pos_near_weights(int4[], float4[], int4[], float4[]);
--DROP FUNCTION search_get_doclist(wordid1 INT, wordid2 INT);
--DROP FUNCTION search_get_doclist(word1 VARCHAR, word2 VARCHAR);
DROP TYPE __t_hits_score CASCADE;
DROP TYPE __t_docid_hits_score CASCADE;
DROP TYPE __t_docid_docname_hits_score CASCADE;

CREATE TYPE __t_hits_score AS (hits int, score real);
CREATE TYPE __t_docid_hits_score AS (docid int, res __t_hits_score);
CREATE TYPE __t_docid_docname_hits_score AS (docid int, docname varchar, hits int, score real);


CREATE OR REPLACE FUNCTION srch_pos_near_weights_oneword(int4[], float4[])
--    RETURNS float4
    RETURNS __t_hits_score
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights_oneword'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights(int4[], float4[], int4[], float4[])
--    RETURNS float4
    RETURNS __t_hits_score
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights(int4[], float4[], int4[], float4[], int4[], float4[])
--    RETURNS float4
    RETURNS __t_hits_score
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights(int4[], float4[], int4[], float4[], int4[], float4[], int4[], float4[])
--    RETURNS float4
    RETURNS __t_hits_score
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights'
    LANGUAGE C IMMUTABLE STRICT;


--	CREATE OR REPLACE FUNCTION search_get_doclist(wordid1 INT, wordid2 INT) RETURNS SETOF __t_docid_hits_score AS
--	$BODY$
--	DECLARE
--	--	rec RECORD;
--		rec __t_docid_hits_score;
--	BEGIN
--		RETURN QUERY SELECT h1.docid, srch_pos_near_weights(h1.positions, h1.weights, h2.positions, h2.weights) as res
--			FROM hyps AS h1, hyps AS h2
--			WHERE h1.wordid=wordid1
--			AND h2.wordid=wordid2 
--			AND h1.docid=h2.docid;
--	--	  ,LEAST(h1.total_weight, h2.total_weight) AS score 
--	END
--	$BODY$
--	LANGUAGE 'plpgsql' ;


-- ##################################################
-- # search 1 word {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_doclist_count(wordid1 INT, doclist INT[]) RETURNS SETOF __t_docid_hits_score AS
$BODY$
DECLARE
--    r RECORD;
    r __t_docid_hits_score;
	counter INT;
BEGIN
	counter = 0;

--	FOR r IN SELECT h1.docid, ROW(h1.length, h1.total_weight) as res
    FOR r IN SELECT h1.docid, srch_pos_near_weights_oneword(h1.positions, h1.weights) as res
		FROM hyps AS h1
		WHERE h1.wordid=wordid1
		AND (doclist IS NULL OR h1.docid=ANY(doclist))
    LOOP
		IF (r.res).hits > 0 THEN
			counter = counter + 1;
		END IF;
        RETURN NEXT r; -- return current row of SELECT
    END LOOP;
	RAISE NOTICE 'count: %', counter;
--	r.docid = 0;
--	r.res = (counter,100000);
--	RETURN NEXT r;
    RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_doclist(word1 VARCHAR, doclist TEXT[]) RETURNS SETOF __t_docid_docname_hits_score AS
$BODY$
DECLARE
	rec __t_docid_docname_hits_score;
	wordid1 INTEGER;
BEGIN
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;

	RETURN QUERY SELECT res.docid, docs.name, (res).hits, (res).score 
		FROM search_get_doclist_count(
			(SELECT wordid FROM dict WHERE word=word1), 
			(SELECT array_agg(docid) FROM docs WHERE name=ANY(doclist))
		) as res 
		LEFT JOIN docs ON docs.docid = res.docid
		WHERE (res).hits > 0
		ORDER BY score desc, docid;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 1 word
-- ##################################################

-- ##################################################
-- # search 2 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_doclist_count(wordid1 INT, wordid2 INT, doclist INT[]) RETURNS SETOF __t_docid_hits_score AS
$BODY$
DECLARE
--    r RECORD;
    r __t_docid_hits_score;
	counter INT;
BEGIN
	counter = 0;

    FOR r IN SELECT h1.docid, srch_pos_near_weights(h1.positions, h1.weights, h2.positions, h2.weights) as res
		FROM hyps AS h1, hyps AS h2
		WHERE h1.wordid=wordid1
		AND h2.wordid=wordid2 
		AND h1.docid=h2.docid
		AND (doclist IS NULL OR h1.docid=ANY(doclist))
    LOOP
		IF (r.res).hits > 0 THEN
			counter = counter + 1;
		END IF;
        RETURN NEXT r; -- return current row of SELECT
    END LOOP;
	RAISE NOTICE 'count: %', counter;
--	r.docid = 0;
--	r.res = (counter,100000);
--	RETURN NEXT r;
    RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_doclist(word1 VARCHAR, word2 VARCHAR, doclist TEXT[]) RETURNS SETOF __t_docid_docname_hits_score AS
$BODY$
DECLARE
	rec __t_docid_docname_hits_score;
	wordid1 INTEGER;
	wordid2 INTEGER;
BEGIN
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;

	RETURN QUERY SELECT res.docid, docs.name, (res).hits, (res).score 
		FROM search_get_doclist_count(
			(SELECT wordid FROM dict WHERE word=word1), 
			(SELECT wordid FROM dict WHERE word=word2),
			(SELECT array_agg(docid) FROM docs WHERE name=ANY(doclist))
		) as res 
		LEFT JOIN docs ON docs.docid = res.docid
		WHERE (res).hits > 0
		ORDER BY score desc, docid;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 2 words
-- ##################################################

-- ##################################################
-- # search 3 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_doclist_count(wordid1 INT, wordid2 INT, wordid3 INT, doclist INT[]) RETURNS SETOF __t_docid_hits_score AS
$BODY$
DECLARE
--    r RECORD;
    r __t_docid_hits_score;
	counter INT;
BEGIN
	counter = 0;

    FOR r IN SELECT h1.docid, srch_pos_near_weights(h1.positions, h1.weights, h2.positions, h2.weights, h3.positions, h3.weights) as res
		FROM hyps AS h1, hyps AS h2, hyps AS h3
		WHERE h1.wordid=wordid1
		AND h2.wordid=wordid2 
		AND h3.wordid=wordid3 
		AND h1.docid=h2.docid
		AND h1.docid=h3.docid
		AND (doclist IS NULL OR h1.docid=ANY(doclist))
    LOOP
		IF (r.res).hits > 0 THEN
			counter = counter + 1;
		END IF;
        RETURN NEXT r; -- return current row of SELECT
    END LOOP;
	RAISE NOTICE 'count: %', counter;
--	r.docid = 0;
--	r.res = (counter,100000);
--	RETURN NEXT r;
    RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_doclist(word1 VARCHAR, word2 VARCHAR, word3 VARCHAR, doclist TEXT[]) RETURNS SETOF __t_docid_docname_hits_score AS
$BODY$
DECLARE
	rec __t_docid_docname_hits_score;
	wordid1 INTEGER;
	wordid2 INTEGER;
	wordid3 INTEGER;
BEGIN
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;
	SELECT wordid INTO wordid3 FROM dict WHERE word=word3;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word3;
	END IF;

	RETURN QUERY SELECT res.docid, docs.name, (res).hits, (res).score 
		FROM search_get_doclist_count(
			(SELECT wordid FROM dict WHERE word=word1), 
			(SELECT wordid FROM dict WHERE word=word2),
			(SELECT wordid FROM dict WHERE word=word3),
			(SELECT array_agg(docid) FROM docs WHERE name=ANY(doclist))
		) as res 
		LEFT JOIN docs ON docs.docid = res.docid
		WHERE (res).hits > 0
		ORDER BY score desc, docid;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 3 words
-- ##################################################

-- ##################################################
-- # search 4 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_doclist_count(wordid1 INT, wordid2 INT, wordid3 INT, wordid4 INT, doclist INT[]) RETURNS SETOF __t_docid_hits_score AS
$BODY$
DECLARE
--    r RECORD;
    r __t_docid_hits_score;
	counter INT;
BEGIN
	counter = 0;

    FOR r IN SELECT h1.docid, srch_pos_near_weights(h1.positions, h1.weights, h2.positions, h2.weights, h3.positions, h3.weights, h4.positions, h4.weights) as res
		FROM hyps AS h1, hyps AS h2, hyps AS h3, hyps AS h4
		WHERE h1.wordid=wordid1
		AND h2.wordid=wordid2 
		AND h3.wordid=wordid3 
		AND h4.wordid=wordid4 
		AND h1.docid=h2.docid
		AND h1.docid=h3.docid
		AND h1.docid=h4.docid
		AND (doclist IS NULL OR h1.docid=ANY(doclist))
    LOOP
		IF (r.res).hits > 0 THEN
			counter = counter + 1;
		END IF;
        RETURN NEXT r; -- return current row of SELECT
    END LOOP;
	RAISE NOTICE 'count: %', counter;
--	r.docid = 0;
--	r.res = (counter,100000);
--	RETURN NEXT r;
    RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_doclist(word1 VARCHAR, word2 VARCHAR, word3 VARCHAR, word4 VARCHAR, doclist TEXT[]) RETURNS SETOF __t_docid_docname_hits_score AS
$BODY$
DECLARE
	rec __t_docid_docname_hits_score;
	wordid1 INTEGER;
	wordid2 INTEGER;
	wordid3 INTEGER;
	wordid4 INTEGER;
BEGIN
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;
	SELECT wordid INTO wordid3 FROM dict WHERE word=word3;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word3;
	END IF;
	SELECT wordid INTO wordid4 FROM dict WHERE word=word4;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word4;
	END IF;

	RETURN QUERY SELECT res.docid, docs.name, (res).hits, (res).score 
		FROM search_get_doclist_count(
			(SELECT wordid FROM dict WHERE word=word1), 
			(SELECT wordid FROM dict WHERE word=word2),
			(SELECT wordid FROM dict WHERE word=word3),
			(SELECT wordid FROM dict WHERE word=word4),
			(SELECT array_agg(docid) FROM docs WHERE name=ANY(doclist))
		) as res 
		LEFT JOIN docs ON docs.docid = res.docid
		WHERE (res).hits > 0
		ORDER BY score desc, docid;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 4 words
-- ##################################################


-- select * from search_get_doclist('proto','je') LIMIT 10;
-- select * from search_get_doclist('qqqqq','je') LIMIT 10;
-- select * from search_get_doclist('je','qqqqq') LIMIT 10;

