\timing on

-- CREATE LANGUAGE plpgsql;

-- DROP FUNCTION srch_pos_near_weights_occurrences(int4[], float4[], int4[], float4[]);
-- DROP FUNCTION search_get_occurrences(docid INT, wordid1 INT, wordid2 INT);
-- DROP FUNCTION search_get_occurrences(docname VARCHAR, word1 VARCHAR, word2 VARCHAR);
DROP TYPE __occurrence_composite_multi CASCADE;

CREATE TYPE __occurrence_composite_multi AS (tstart real, tend real, score real);

CREATE OR REPLACE FUNCTION srch_pos_near_weights_occurrences_oneword(int4[], float4[])
    RETURNS SETOF __occurrence_composite_multi
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights_occurrences_oneword'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights_occurrences(int4[], float4[], int4[], float4[])
    RETURNS SETOF __occurrence_composite_multi
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights_occurrences'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights_occurrences(int4[], float4[], int4[], float4[], int4[], float4[])
    RETURNS SETOF __occurrence_composite_multi
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights_occurrences'
    LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION srch_pos_near_weights_occurrences(int4[], float4[], int4[], float4[], int4[], float4[], int4[], float4[])
    RETURNS SETOF __occurrence_composite_multi
    AS '/home/miso/projects/speech_search/postgresql/searchPositionsNear/libpgSrchPosNear.so', 'c_srch_pos_near_weights_occurrences'
    LANGUAGE C IMMUTABLE STRICT;

-- ##################################################
-- # search 1 word {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_occurrences(docid INT, wordid1 INT) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	rec RECORD;
	occ __occurrence_composite_multi;
	counter INT;
BEGIN
	SELECT h1.positions as p1, h1.weights as w1
	INTO rec
	FROM hyps AS h1
	WHERE h1.docid=docid AND h1.wordid=wordid1;

	counter = 0;
	FOR occ IN SELECT * FROM srch_pos_near_weights_occurrences_oneword(rec.p1, rec.w1)
	LOOP
		counter = counter + 1;
		RETURN NEXT occ;
	END LOOP;
	RAISE NOTICE 'count: %', counter;
--	occ.tstart = -1;
--	occ.tend = counter;
--	occ.score = 100000;
--	RETURN NEXT occ;
	RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_occurrences(docname VARCHAR, word1 VARCHAR) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	doc_id INTEGER;
	wordid1 INTEGER;
BEGIN
	SELECT docid INTO doc_id FROM docs WHERE name=docname;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Document "%" not found', docname;
	END IF;
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;

	RETURN QUERY SELECT * FROM search_get_occurrences(
		doc_id,
		wordid1
	) ORDER BY score DESC;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 1 word
-- ##################################################

-- ##################################################
-- # search 2 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_occurrences(docid INT, wordid1 INT, wordid2 INT) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	rec RECORD;
	occ __occurrence_composite_multi;
	counter INT;
BEGIN
	SELECT h1.positions as p1, h1.weights as w1, h2.positions as p2, h2.weights as w2
	INTO rec
	FROM hyps AS h1, hyps AS h2
	WHERE h1.docid=docid AND h2.docid=docid AND h1.wordid=wordid1 AND h2.wordid=wordid2; 

	counter = 0;
	FOR occ IN SELECT * FROM srch_pos_near_weights_occurrences(rec.p1, rec.w1, rec.p2, rec.w2)
	LOOP
		counter = counter + 1;
		RETURN NEXT occ;
	END LOOP;
	RAISE NOTICE 'count: %', counter;
--	occ.tstart = -1;
--	occ.tend = counter;
--	occ.score = 100000;
--	RETURN NEXT occ;
	RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_occurrences(docname VARCHAR, word1 VARCHAR, word2 VARCHAR) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	doc_id INTEGER;
	wordid1 INTEGER;
	wordid2 INTEGER;
BEGIN
	SELECT docid INTO doc_id FROM docs WHERE name=docname;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Document "%" not found', docname;
	END IF;
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;

	RETURN QUERY SELECT * FROM search_get_occurrences(
		doc_id,
		wordid1,
		wordid2
	) ORDER BY score DESC;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 2 words
-- ##################################################

-- ##################################################
-- # search 3 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_occurrences(docid INT, wordid1 INT, wordid2 INT, wordid3 INT) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	rec RECORD;
	occ __occurrence_composite_multi;
	counter INT;
BEGIN
	SELECT h1.positions as p1, h1.weights as w1, 
		h2.positions as p2, h2.weights as w2, 
		h3.positions as p3, h3.weights as w3
	INTO rec
	FROM hyps AS h1, hyps AS h2, hyps AS h3
	WHERE h1.docid=docid 
		AND h2.docid=docid 
		AND h3.docid=docid 
		AND h1.wordid=wordid1 
		AND h2.wordid=wordid2 
		AND h3.wordid=wordid3; 

	counter = 0;
	FOR occ IN SELECT * FROM srch_pos_near_weights_occurrences(rec.p1, rec.w1, rec.p2, rec.w2, rec.p3, rec.w3)
	LOOP
		counter = counter + 1;
		RETURN NEXT occ;
	END LOOP;
	RAISE NOTICE 'count: %', counter;
--	occ.tstart = -1;
--	occ.tend = counter;
--	occ.score = 100000;
--	RETURN NEXT occ;
	RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_occurrences(docname VARCHAR, word1 VARCHAR, word2 VARCHAR, word3 VARCHAR) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	doc_id INTEGER;
	wordid1 INTEGER;
	wordid2 INTEGER;
	wordid3 INTEGER;
BEGIN
	SELECT docid INTO doc_id FROM docs WHERE name=docname;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Document "%" not found', docname;
	END IF;
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;
	SELECT wordid INTO wordid3 FROM dict WHERE word=word3;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word3;
	END IF;

	RETURN QUERY SELECT * FROM search_get_occurrences(
		doc_id,
		wordid1,
		wordid2,
		wordid3
	) ORDER BY score DESC;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 3 words
-- ##################################################

-- ##################################################
-- # search 4 words {{{
-- ##################################################
CREATE OR REPLACE FUNCTION search_get_occurrences(docid INT, wordid1 INT, wordid2 INT, wordid3 INT, wordid4 INT) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	rec RECORD;
	occ __occurrence_composite_multi;
	counter INT;
BEGIN
	SELECT h1.positions as p1, h1.weights as w1, 
		h2.positions as p2, h2.weights as w2, 
		h3.positions as p3, h3.weights as w3,
		h4.positions as p4, h4.weights as w4
	INTO rec
	FROM hyps AS h1, hyps AS h2, hyps AS h3, hyps AS h4
	WHERE h1.docid=docid 
		AND h2.docid=docid 
		AND h3.docid=docid 
		AND h4.docid=docid 
		AND h1.wordid=wordid1 
		AND h2.wordid=wordid2 
		AND h3.wordid=wordid3
		AND h4.wordid=wordid4; 

	counter = 0;
	FOR occ IN SELECT * FROM srch_pos_near_weights_occurrences(rec.p1, rec.w1, rec.p2, rec.w2, rec.p3, rec.w3, rec.p4, rec.w4)
	LOOP
		counter = counter + 1;
		RETURN NEXT occ;
	END LOOP;
	RAISE NOTICE 'count: %', counter;
--	occ.tstart = -1;
--	occ.tend = counter;
--	occ.score = 100000;
--	RETURN NEXT occ;
	RETURN;
END
$BODY$
LANGUAGE 'plpgsql' ;


CREATE OR REPLACE FUNCTION search_get_occurrences(docname VARCHAR, word1 VARCHAR, word2 VARCHAR, word3 VARCHAR, word4 VARCHAR) RETURNS SETOF __occurrence_composite_multi AS
$BODY$
DECLARE
	doc_id INTEGER;
	wordid1 INTEGER;
	wordid2 INTEGER;
	wordid3 INTEGER;
	wordid4 INTEGER;
BEGIN
	SELECT docid INTO doc_id FROM docs WHERE name=docname;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Document "%" not found', docname;
	END IF;
	SELECT wordid INTO wordid1 FROM dict WHERE word=word1;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word1;
	END IF;
	SELECT wordid INTO wordid2 FROM dict WHERE word=word2;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word2;
	END IF;
	SELECT wordid INTO wordid3 FROM dict WHERE word=word3;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word3;
	END IF;
	SELECT wordid INTO wordid4 FROM dict WHERE word=word4;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Word "%" not found', word4;
	END IF;

	RETURN QUERY SELECT * FROM search_get_occurrences(
		doc_id,
		wordid1,
		wordid2,
		wordid3,
		wordid4
	) ORDER BY score DESC;
END
$BODY$
LANGUAGE 'plpgsql' ;
-- ##################################################
-- # }}} search 4 words
-- ##################################################




--SELECT * FROM search_get_occurrences(295, 44791, 226577) ORDER by score DESC LIMIT 10;
--SELECT * FROM search_get_occurrences(295, 224796, 60657) ORDER by score DESC LIMIT 10;

--SELECT * FROM search_get_occurrences('ISS-2008-10-31-5', 'fourierova', 'transformace') ORDER by score DESC LIMIT 10;
